﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace LP_MineSweeper {
	public class Cell : Button {
		// static members (read by all Cells)
		public static Color neutralColor = SystemColors.Control;
		public static Bitmap BitmapMine;
		public static Bitmap BitmapFlag;
		public const int CellSize = 42;
		private static readonly Color[] textColors = new Color[]{Color.White, Color.Blue, Color.Green, 
			Color.Red, Color.Magenta, Color.Blue, Color.Green, Color.Red, Color.Magenta };
		private static readonly Font font = new Font("Verdana", CellSize / 3f, FontStyle.Bold);

		// instance members
		private readonly Game game;
		private readonly int row;
		private readonly int col;
		private int dangerLevel; // number of neighboring mines
		private bool givenAway;	 // mine position can be given away when aborting a game

		public Cell(Game game, int row, int col) {
			this.game = game;
			this.row = row;
			this.col = col;
			Width = CellSize;
			Height = CellSize;
			Font = font;
			BackgroundImageLayout = ImageLayout.Center;
			MouseDown += button_Click;
			Paint += button_Paint;	// so we can give an initial 3D effect to buttons
		}
		public bool IsRevealed { get; set; }    // player has left clicked, dangerLevel count (or mine) is visible now
		public bool IsMine { get; set; }        // cell holds a mine
		public bool IsFlagged { get; set; }     // player guesses a mine is present
		public bool IsKnownToBeSafe { get { return dangerLevel == 0 && IsRevealed; } }

		private void log(string text) {
			game.AddToLog?.Invoke(text);
		}

		public void IncrementDangerLevel() {
			dangerLevel++;
		}

		public override string ToString() {
			string s = "cell[" + row + "," + col + "]";
			if (IsMine) s += " = mine";
			return s;
		}

		public void button_Click(object sender, EventArgs e) {
			log("Click " + this);
			if (MouseButtons == MouseButtons.Right) {
				if (!IsRevealed) {
					IsFlagged = !IsFlagged;     // toggle flag
					BackgroundImage = IsFlagged ? BitmapFlag : null;
					ForeColor = Color.Black;
					game.UpdateGame(null);		// report click for counter update (no cell, no sound)
				}
			} else {
				if (!IsRevealed) {
					Reveal();
					game.UpdateGame(this);  // report click for flood fill, counter update, and game termination check
				}
			}
		}

		// a separate method to reveal the cell's content, called by the game's floodfill method.
		public void Reveal() {
			IsRevealed = true;
			// reveal cell state by changing button properties
			MouseDown -= button_Click;		// won't be clicked again
			FlatStyle = FlatStyle.Flat;
			FlatAppearance.BorderSize = 0;
			if (IsMine) {
				// show mine image on red background
				BackColor = Color.Red;
				BackgroundImage = BitmapMine;	
				Text = "";
			} else {
				// show colored danger level digit (empty when zero)
				BackColor = neutralColor;
				ForeColor = textColors[dangerLevel];	
				BackgroundImage = null;
				Text = dangerLevel == 0 ? "" : dangerLevel.ToString();
			}
			// no more color change any more when hovering
			FlatAppearance.MouseOverBackColor = BackColor;  
		}

		// show strong hint when mine present (used by Game.Abort command)
		public void GiveAway() {
			if (IsMine) {
				givenAway = true;
				Invalidate();
			}
		}

		// add 3D effect to unrevealed cell
		private void button_Paint(object sender, PaintEventArgs e) {
			if (!IsRevealed) {
				int wid = 3;
				Color color = SystemColors.ControlLightLight;
				if (givenAway && IsMine) color = Color.Red;
				ControlPaint.DrawBorder(e.Graphics, ClientRectangle,
					color, wid, ButtonBorderStyle.Outset,
					color, wid, ButtonBorderStyle.Outset,
					color, wid, ButtonBorderStyle.Outset,
					color, wid, ButtonBorderStyle.Outset);
			}
		}
	}
}
