﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Media;
using System.Reflection;
using System.Windows.Forms;

namespace LP_MineSweeper {
	public partial class GameForm : Form {
		private const string VERSION = "1.1";
		private int nRows;
		private int nCols;
		private int nMines;
		private bool debugFlag;
		private bool soundFlag;
		private SoundPlayer explosion;
		private Game game;

		public GameForm() {
			InitializeComponent();
			Text=Text.Replace("#", VERSION);
			if (Properties.Settings.Default.DebugFlag) ((ToolStripMenuItem)debugToolStripMenuItem).PerformClick();
			if (Properties.Settings.Default.SoundFlag) ((ToolStripMenuItem)soundToolStripMenuItem).PerformClick();
			string index = Properties.Settings.Default.GameParmsIndex;
			foreach (ToolStripItem tsi in sizeToolStripMenuItem.DropDownItems) {
				if ((string)tsi.Tag==index) ((ToolStripMenuItem)tsi).PerformClick();
			}
		}

		#region support methods
		// add one line of text to the debug ListBox
		public void log(string text) {
			lb.Items.Add(text);
			lb.TopIndex = lb.Items.Count - 1; // scroll up
		}
		public Bitmap GetEmbeddedImage(string filename) {
			try {
				Assembly assembly = Assembly.GetEntryAssembly();
				Stream myStream = assembly.GetManifestResourceStream("LP_MineSweeper.Resources." + filename);
				return new Bitmap(myStream);
			} catch (Exception exc) {
				log("Exception in GetEmbeddedImage(" + filename + "): " + exc.Message);
				return null;
			}
		}

		public SoundPlayer GetEmbeddedSound(string filename) {
			try {
				Assembly assembly = Assembly.GetEntryAssembly();
				Stream myStream = assembly.GetManifestResourceStream("LP_MineSweeper.Resources." + filename);
				SoundPlayer player = new SoundPlayer(myStream);
				player.Load();
				return player;
			} catch (Exception exc) {
				log("Exception in GetEmbeddedSound(" + filename + "): " + exc.Message);
				return null;
			}
		}
		#endregion

		#region event handlers
		private void GameForm_Load(object sender, EventArgs e) {
			Cell.BitmapMine = GetEmbeddedImage("mine32.png");
			Cell.BitmapFlag = GetEmbeddedImage("flag32.png");
			explosion = GetEmbeddedSound("explosion.wav");
		}

		private void GameForm_FormClosing(object sender, FormClosingEventArgs e) {
			Properties.Settings.Default.Save();
		}

		[System.Diagnostics.CodeAnalysis.SuppressMessage("Style", "IDE0017:Simplify object initialization", Justification = "Silly suggestion")]
		private void startToolStripMenuItem_Click(object sender, EventArgs e) {
			game?.Stop();
			ssMines.Value = "";
			labStatus.Text = "";
			ssTime.Value = "";
			game=new Game(log, nRows, nCols, nMines);
			game.ShowTime = (time) => ssTime.Value= time>999.9 ? " OFL" : string.Format("{0:N1}", time);
			game.ShowHiddenMineCount = (count) => ssMines.Value = count.ToString();
			game.ShowStatus = (text) => { labStatus.Text = text + ".  Play again?"; 
				if (soundFlag && !text.Contains("ad luck")) SystemSounds.Hand.Play(); };
			game.PlaySound = (sound) => { if (soundFlag && sound == "explosion") explosion.Play(); };
			game.UpdateGame(null);
			setupTablePanel();
		}

		private void abortGameToolStripMenuItem_Click(object sender, EventArgs e) {
			game?.Abort(true);
		}

		private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
			Application.Exit();
		}

		private void _10minesIn9X9BoardToolStripMenuItem_Click(object sender, EventArgs e) {
			setGameParms(10, 9, 9, "0");
		}

		private void _40minesIn16X16BoardToolStripMenuItem_Click(object sender, EventArgs e) {
			setGameParms(40, 16, 16, "1");
		}

		private void _99minesIn30X16BoardToolStripMenuItem_Click(object sender, EventArgs e) {
			setGameParms(99, 16, 30, "2");
		}

		private void setGameParms(int nMines, int nRows, int nCols, string index) {
			tablePanel.Visible = false;
			game?.Stop();
			this.nMines = nMines;
			this.nRows = nRows;
			this.nCols = nCols;
			Properties.Settings.Default.GameParmsIndex = index;
			foreach (ToolStripItem tsi in sizeToolStripMenuItem.DropDownItems) {
				((ToolStripMenuItem)tsi).Checked = (string)tsi.Tag == index;
			}
			resizeForm();
		}

		// adapt form size to number of cells (and optional presence of debug ListBox)
		// then centralize on the screen while taking into account the taskbar (if any).
		// for multi-monitor systems, we keep the Start button on its current screen
		private void resizeForm() {
			Rectangle area = Screen.FromControl(btnStart).WorkingArea;
			Height = tablePanel.Top + nRows * Cell.CellSize + 80;
			int wid = tablePanel.Left + nCols * Cell.CellSize + nCols + 40;
			if (debugFlag) wid += lb.Width + 20;
			Width = wid;
			Top = area.Top + (area.Height - Height) / 2;
			Left = area.Left + (area.Width - Width) / 2;
		}

		private void soundToolStripMenuItem_Click(object sender, EventArgs e) {
			soundFlag = soundToolStripMenuItem.Checked;
			Properties.Settings.Default.SoundFlag = soundFlag;

		}

		private void debugToolStripMenuItem_Click(object sender, EventArgs e) {
			debugFlag = debugToolStripMenuItem.Checked;
			Properties.Settings.Default.DebugFlag = debugFlag;
			lb.Visible = debugFlag;
			resizeForm();
		}

		private void aboutToolStripMenuItem_Click(object sender, EventArgs e) {
			using (AboutBox1 about = new AboutBox1()) {
				about.Version = VERSION;
				about.ShowDialog();
			}
		}

		private void millionDollarMinesweeperToolStripMenuItem_Click(object sender, EventArgs e) {
			Process.Start("http://www.minesweeper.info/articles/MillionDollarMinesweeper.pdf", null);
		}

		private void mineSweeperSiteToolStripMenuItem_Click(object sender, EventArgs e) {
			Process.Start("https://minesweepergame.com/");
		}
		#endregion

		// stuff the cells into the TableLayoutPanel so the player can see them
		// row and column styles must be set first to obtain fixed sizes
		private void setupTablePanel() {
			tablePanel.Visible = false;		// hide while constructing new board
			tablePanel.SuspendLayout();		// dramatic speed increase!
			tablePanel.ColumnCount = nCols;
			tablePanel.RowCount = nRows;
			tablePanel.Controls.Clear();
			// set all rows and columns to fixed size
			tablePanel.ColumnStyles.Clear();
			for (int c = 0; c < nCols; c++) {
				tablePanel.ColumnStyles.Add(new ColumnStyle(SizeType.Absolute, Cell.CellSize));
			}
			tablePanel.RowStyles.Clear();
			for (int r = 0; r < nRows; r++) {
				tablePanel.RowStyles.Add(new RowStyle(SizeType.Absolute, Cell.CellSize));
			}
			// now insert the cells
			for (int r = 0; r < nRows; r++) {
				for (int c = 0; c < nCols; c++) {
					tablePanel.Controls.Add(game.GetCell(r, c), c, r);
				}
			}
			tablePanel.ResumeLayout();
			tablePanel.Visible = true;
			log("size=" + tablePanel.Size);
		}
	}
}
